﻿using System;
using System.Collections.Generic;
using System.Web;
using System.IO;
using System.Configuration;

namespace MDK
{
    /// <summary>
    /// 設定ファイル(Merchant.config)を読み込み、保持するクラス
    /// </summary>
    public class MerchantConf
    {
        /// <summary>
        /// 設定ファイル名
        /// </summary>
        private static String fileName = "MerchantConf.xml";

        /// <summary>
        /// 自インスタンス
        /// </summary>
        private static MerchantConf info = null;

        /// <summary>
        /// ログ出力用インスタンス
        /// </summary>
        private static readonly log4net.ILog logger = 
            log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        /// <summary>
        /// マーチャントID
        /// </summary>
        public String merchantID;

        /// <summary>
        /// カード売上フラグ
        /// </summary>
        public String cardCaptureFlag;

        /// <summary>
        /// シード
        /// </summary>
        public String seed;

        /// <summary>
        /// 決済方法
        /// </summary>
        public String settlementType;

        /// <summary>
        /// VT Web 暗号鍵取得URL
        /// </summary>
        public String vtWebRegistUrl;

        /// <summary>
        /// VT Web 決済URL
        /// </summary>
        public String vtWebSettlementUrl;

        /// <summary>
        /// 決済完了後戻りURL
        /// </summary>
        public String finishPaymentReturnUrl;

        /// <summary>
        /// 未決済時戻りURL
        /// </summary>
        public String unfinishPaymentReturnUrl;

        /// <summary>
        /// 決済エラー時戻りURL
        /// </summary>
        public String errorPaymentReturnUrl;

        /// <summary>
        /// 決済結果通知先URL
        /// </summary>
        public String finishPaymentAccessUrl;

        /// <summary>
        /// 言語選択可否フラグ
        /// </summary>
        public String langEnableFlag;

        /// <summary>
        /// 使用言語
        /// </summary>
        public String lang;

        /// <summary>
        /// ダミー取引フラグ
        /// </summary>
        public String dummyPaymentFlag;

        /// <summary>
        /// モバイルEdy用ショップ名
        /// </summary>
        public String shopName;

        /// <summary>
        /// Suica用商品名
        /// </summary>
        public String screenTitle;

        /// <summary>
        /// 請求内容
        /// </summary>
        public String contents;

        /// <summary>
        /// 請求内容カナ
        /// </summary>
        public String contentsKana;

        /// <summary>
        /// PayNowID使用フラグ
        /// </summary>
        public String paynowidFlag;

        /// <summary>
        /// 会員ID
        /// </summary>
        public String accountId;

        /// <summary>
        /// 銀行振込決済の登録時振込番号
        /// </summary>
        public String vaccEntryTransferNumber;

        /// <summary>
        /// 銀行振込決済の振込期限
        /// </summary>
        public String vaccTransferExpiredDate;

        /// <summary>
        /// 銀行振込決済の口座管理方式
        /// </summary>
        public String vaccAccountManageType;

        /// <summary>
        /// 銀行振込決済の支店コード
        /// </summary>
        public String vaccBranchCode;

        /// <summary>
        /// 銀行振込決済の口座番号
        /// </summary>
        public String vaccAccountNumber;

        /// <summary>
        /// 銀行口座関連付け拒否フラグ
        /// </summary>
        public String vaccRejectBankAccount;

        /// <summary>
        /// VT Webに送信するダミー商品ID
        /// </summary>
        public String dummyCommodityId;

        /// <summary>
        /// VT Webに送信するダミーJAN_CODE
        /// </summary>
        public String dummyCommodityJanCode;

        /// <summary>
        /// コンストラクタ。外部からの生成を防止
        /// </summary>
        private MerchantConf()
        {
        }

        /// <summary>
        /// インスタンスを取得
        /// </summary>
        /// <returns>マーチャント情報</returns>
        public static MerchantConf getInfo()
        {
            if (info == null)
            {
                String rootPath = System.Web.Hosting.HostingEnvironment.MapPath("~");
                String filePath = rootPath + @"\" + fileName;

                if (!File.Exists(filePath))
                {
                    throw new Exception(filePath + " が存在しません");
                }

                //＜XMLファイルから読み込む＞
                //XmlSerializerオブジェクトの作成
                System.IO.FileStream fs = null;
                try
                {
                    System.Xml.Serialization.XmlSerializer serializer =
                        new System.Xml.Serialization.XmlSerializer(typeof(MerchantConf));
                    //ファイルを開く
                    fs = new System.IO.FileStream(filePath, System.IO.FileMode.Open, FileAccess.Read);
                    //XMLファイルから読み込み、逆シリアル化する
                    info = (MerchantConf)serializer.Deserialize(fs);
                }
                catch
                {
                    throw new Exception(filePath + " の読み込みエラー");
                }
                finally
                {
                    //閉じる
                    if (fs != null)
                    {
                        fs.Close();
                    }
                }
            }
            return info;
        }

    }
}