﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Collections;

namespace MDK
{
    /// <summary>
    /// 3G-Webからの受信情報を保持するクラス。
    /// </summary>
    public class PostDataBox
    {
        /// <summary>
        /// 自インスタンス。
        /// </summary>
        private static PostDataBox dataBox = new PostDataBox();

        /// <summary>
        /// 暗号鍵保持ハッシュ。
        /// </summary>
        private Hashtable box = new Hashtable();

        
        /// <summary>
        /// コンストラクタ。外部からインスタンス生成を禁止。
        /// </summary>
        private PostDataBox()
        {
        }

        /// <summary>
        /// 自インスタンスを返す。
        /// </summary>
        /// <returns>自インスタンス</returns>
        public static PostDataBox getInstance()
        {
            return dataBox;
        }

        /// <summary>
        /// VTWebからのResponse(暗号鍵など)を設定する。
        /// </summary>
        /// <param name="key">取引ID</param>
        /// <param name="hash">VTWebからのResponse(暗号鍵など)を保持したHashtable</param>
        public void setBox(String key, Hashtable hash)
        {
            if (box.ContainsKey(key))
            {
                box[key] = hash;
            }
            else
            {
                box.Add(key, hash);
            }
        }

        /// <summary>
        /// VTWebからのResponse(暗号鍵など)を取得する。
        /// </summary>
        /// <param name="key">取引ID</param>
        /// <returns>VTWebからのResponse(暗号鍵など)を保持したHashtable</returns>
        public Hashtable getBox(String key)
        {
            if (String.IsNullOrEmpty(key))
                return null;

            return (Hashtable)box[key];
        }

        /// <summary>
        /// 暗号鍵保持ハッシュを返却する。
        /// </summary>
        /// <returns>暗号鍵保持ハッシュ</returns>
        public Hashtable getBox()
        {
            return box;
        }
    }
}