﻿using System;
using System.Collections.Generic;
using System.Web;


namespace MDK
{
    /// <summary>
    /// 決済結果チェッククラス
    /// </summary>
    public class ResultCheck
    {
        /// <summary>
        /// ログ出力用インスタンス
        /// </summary>
        private static readonly log4net.ILog logger =
            log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        /// <summary>
        /// コンストラクタ
        /// </summary>
        public ResultCheck()
        {
        }

        /// <summary>
        /// VTWからのRequestの内容確認
        /// </summary>
        /// <param name="databean">結果Bean</param>
        /// <returns>確認結果</returns>
        public static bool resultCheck(ResultData databean)
        {
            //どれか一つでもnullがあったらエラー
            if (!isValid(databean.OrderId) ||
                !isValid(databean.MStatus) ||
                !isValid(databean.VResultCode) )
            {
                return false;
            }
            else
            {
                logger.Info("リクエストにnullはなし");
                return true;
            }

            /*****************************************************
             *  その他のパラメータは、VT Webで入力した値が戻される。
             *  戻すか否かの判定は、マーチャント付加情報マスタの
             *  REQUIRE_PERSONAL_INFO_FOR_ECによって判断される
             *****************************************************/
        }

        /// <summary>
        /// 値保持チェック
        /// </summary>
        /// <param name="str">チェック対象文字列</param>
        /// <returns>チェック結果</returns>
        private static bool isValid(String str)
        {
            if (str == null || str.Equals("null"))
            {
                return false;
            }
            else
                return true;
        }

        /// <summary>
        /// VWT側とブラウザ側の整合性チェック
        /// </summary>
        /// <param name="hash">VT WebのRequestの内容が格納されたHashtable</param>
        /// <param name="databean">ブラウザのリクエストの内容が格納されたBean</param>
        /// <returns>チェック結果</returns>
        public static bool resultCrossCheck(System.Collections.Hashtable hash, ResultData databean)
        {
            bool complete;

            String mStatus = StringUtil.emptyString((String)hash[Constants.VTWRES_M_STATUS]);
            String vResultCode = StringUtil.emptyString((String)hash[Constants.VTWRES_V_RESULT_CODE]);

            logger.Info("stored orderId:" + databean.OrderId);
            logger.Info("stored mStatus:" + mStatus);
            logger.Info("stored vResultCode:" + vResultCode);

            complete = isMatch(mStatus,databean.MStatus);
            complete &= isMatch(vResultCode,databean.VResultCode);

            return complete;

            /*****************************************************
             *  その他のパラメータは、VT Webで入力した値が戻される。
             *  戻すか否かの判定は、マーチャント付加情報マスタの
             *  REQUIRE_PERSONAL_INFO_FOR_ECによって判断される
             *****************************************************/
        }

        /// <summary>
        /// 2つの文字列が一致しているかどうか比較
        /// ※互いにnullの場合は false とする
        /// </summary>
        /// <param name="str1">比較対象1</param>
        /// <param name="str2">比較対象2</param>
        /// <returns>結果</returns>
        public static bool isMatch(String str1, String str2)
        {
            bool result = false;
            if(str1 != null && str1.Equals(str2))
            {
                result = true;
            }
            return result;
        }
    }
}