﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Collections;
using System.Text;
using System.Net;
using System.IO;

namespace MDK
{
    /// <summary>
    /// 3G-Webへの送信クラス。
    /// </summary>
    public class SendPostData
    {
        /// <summary>
        /// コンストラクタ
        /// </summary>
        public SendPostData()
        {
            hash.Add(KeyDef.PAYNOWID_FLAG, "");
            hash.Add(KeyDef.ACCOUNT_ID, "");
            hash.Add(KeyDef.AMOUNT, "");
            hash.Add(KeyDef.SHIPPING_AMOUNT, "");
            hash.Add(KeyDef.COMMODITY_AMOUNT, "");
            hash.Add(KeyDef.CARD_CAPTURE_FLAG, "");
            hash.Add(KeyDef.VACC_ENTRY_TRANSFER_NUMBER, "");
            hash.Add(KeyDef.VACC_TRANSFER_EXPIRED_DATE, "");
            hash.Add(KeyDef.VACC_ACCOUNT_MANAGE_TYPE, "");
            hash.Add(KeyDef.VACC_BRANCH_CODE, "");
            hash.Add(KeyDef.VACC_ACCOUNT_NUMBER, "");
            hash.Add(KeyDef.VACC_REJECT_BANK_ACCOUNT, "");
            hash.Add(KeyDef.CONTENTS_KANA, "");
            hash.Add(KeyDef.CONTENTS, "");
            hash.Add(KeyDef.FINISH_PAYMENT_RETURN_URL, "");
            hash.Add(KeyDef.UNFINISH_PAYMENT_RETURN_URL, "");
            hash.Add(KeyDef.ERROR_PAYMENT_RETURN_URL, "");
            hash.Add(KeyDef.FINISH_PAYMENT_ACCESS_URL, "");
            hash.Add(KeyDef.LANG_ENABLE_FLAG, "");
            hash.Add(KeyDef.LANG, "");
            hash.Add(KeyDef.DUMMY_PAYMENT_FLAG, "");
            hash.Add(KeyDef.ORICO_ORDER_NO, "");
            hash.Add(KeyDef.SHIPPING_ZIP_CODE, "");
            hash.Add(KeyDef.MERCHANT_ID, "");
            hash.Add(KeyDef.MERCHANTHASH, "");
            hash.Add(KeyDef.ORDER_ID, "");
            hash.Add(KeyDef.SCREEN_TITLE, "");
            hash.Add(KeyDef.SESSION_ID, "");
            hash.Add(KeyDef.SETTLEMENT_SUB_TYPE, "");
            hash.Add(KeyDef.SETTLEMENT_TYPE, "");
            hash.Add(KeyDef.SHOP_NAME, "");
            hash.Add(KeyDef.TIMELIMIT_OF_CANCEL, "");
        }

        /// <summary>
        /// ログ出力用インスタンス
        /// </summary>
        private static readonly log4net.ILog logger = 
            log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        /// <summary>
        /// 送信データ保持ハッシュ
        /// </summary>
        private Hashtable hash = new Hashtable();

        /// <summary>
        /// 商品情報リスト
        /// </summary>
        private List<CommodityDetail> listCommodityDetail = null;

        /// <summary>
        /// ハッシュから情報を削除する。
        /// </summary>
        /// <param name="key">削除情報のキー</param>
        public void removeHash(String key)
        {
            if(hash.ContainsKey(key)){
                hash.Remove(key);
            }
        }

        /// <summary>
        /// ハッシュに情報を追加する。
        /// </summary>
        /// <param name="key">追加情報のキー</param>
        /// <param name="value">追加情報</param>
        public void AddHash(String key, String value)
        {
            if (hash.ContainsKey(key))
            {
                hash[key] = value;
            }
            else
            {
                hash.Add(key, value);
            }
        }

        /// <summary>
        /// ハッシュから情報を取得する
        /// </summary>
        /// <param name="key">情報のキー</param>
        /// <returns>キーに紐付いた情報</returns>
        public String GetHashValue(String key)
        {
            if (hash.ContainsKey(key))
            {
                return hash[key].ToString();
            }
            else
            {
                return String.Empty;
            }
        }

        /// <summary>
        /// ハッシュ内膳情報を削除する
        /// </summary>
        public void ClearHash()
        {
            hash.Clear();
        }

        /// <summary>
        /// 商品情報を設定する
        /// </summary>
        /// <param name="list">商品情報</param>
        public void setListCommodityDetail(List<CommodityDetail> list)
        {
            this.listCommodityDetail = list;
        }

        /// <summary>
        /// 送料合計
        /// </summary>
        public int ShippingAmount { get; set; }

        /// <summary>
        /// 送信情報を文字列で取得する
        /// </summary>
        /// <param name="sessionId">セッションID</param>
        /// <param name="orderId">注文ID</param>
        /// <returns>送信情報</returns>
        public String getPostDataString(String sessionId, String orderId)
        {
            addInfomation();
            
            // Commodityの内容を出力
            StringBuilder sbId = new StringBuilder();
            StringBuilder sbJan = new StringBuilder();
            StringBuilder sbName = new StringBuilder();
            StringBuilder sbUnit = new StringBuilder();
            StringBuilder sbNum = new StringBuilder();

            // AMOUNT
            int amount = 0;
            MerchantConf info = MerchantConf.getInfo();

            foreach(CommodityDetail commodity in listCommodityDetail)
            {
                sbId.Append(String.Format("{0}={1}&", "COMMODITY_ID",
                    (StringUtil.isNullStr(commodity.commodityId)) ? info.dummyCommodityId : commodity.commodityId));
                sbJan.Append(String.Format("{0}={1}&", "JAN_CODE",
                    (StringUtil.isNullStr(commodity.janCode)) ? info.dummyCommodityJanCode : commodity.janCode));
                sbName.Append(String.Format("{0}={1}&", "COMMODITY_NAME", HttpUtility.UrlEncode(commodity.commodityName)));
                sbUnit.Append(String.Format("{0}={1}&", "COMMODITY_UNIT", commodity.commodityUnit));
                sbNum.Append(String.Format("{0}={1}&", "COMMODITY_NUM", commodity.commodityNum));

                amount += commodity.commodityUnit * commodity.commodityNum;
            }
            
            ////////////////////////////////////////////////
            // 必須の共通項目

            // 商品合計金額
            AddHash(KeyDef.COMMODITY_AMOUNT, amount.ToString());
            // 送料合計
            AddHash(KeyDef.SHIPPING_AMOUNT, ShippingAmount.ToString());

            amount += ShippingAmount;
            // 購入金額
            AddHash(KeyDef.AMOUNT, amount.ToString());
            // SessionID
            AddHash(KeyDef.SESSION_ID, sessionId);
            // 取引ID
            AddHash(KeyDef.ORDER_ID, orderId);
            // マーチャント生成ハッシュ値
            String type = hash[KeyDef.SETTLEMENT_TYPE].ToString();
            HashCodeCreater4Merchant creater = new HashCodeCreater4Merchant();
		    String hashCode = creater.getHash(info.merchantID, type, orderId, amount.ToString());
            AddHash(KeyDef.MERCHANTHASH, hashCode);

            // 決済完了後戻りURL
            String backUrl = info.finishPaymentReturnUrl;
            if (!StringUtil.isNullStr(backUrl))
            {
                AddHash(KeyDef.FINISH_PAYMENT_RETURN_URL, backUrl);
            }

            // 未決済時戻りURL
            String unfinishPaymentReturnUrl = info.unfinishPaymentReturnUrl;
            if (!StringUtil.isNullStr(unfinishPaymentReturnUrl))
            {
                AddHash(KeyDef.UNFINISH_PAYMENT_RETURN_URL, unfinishPaymentReturnUrl);
            }

            // 決済エラー時戻りURL
            String errorPaymentReturnUrl = info.errorPaymentReturnUrl;
            if (!StringUtil.isNullStr(errorPaymentReturnUrl))
            {
                AddHash(KeyDef.ERROR_PAYMENT_RETURN_URL, errorPaymentReturnUrl);
            }

            // 決済完了報告URL
            String finishPaymentAccessUrl = info.finishPaymentAccessUrl;
            if (!StringUtil.isNullStr(finishPaymentAccessUrl))
            {
                AddHash(KeyDef.FINISH_PAYMENT_ACCESS_URL, finishPaymentAccessUrl);
            }

            // 言語選択可否フラグ
            String langEnableFlag = info.langEnableFlag;
            if (!StringUtil.isNullStr(backUrl))
            {
                AddHash(KeyDef.LANG_ENABLE_FLAG, langEnableFlag);
            }

            // 使用言語
            String lang = info.lang;
            if (!StringUtil.isNullStr(backUrl))
            {
                AddHash(KeyDef.LANG, lang);
            }

            // ダミー取引フラグ
            String dummyPaymentFlag = info.dummyPaymentFlag;
            if (!StringUtil.isNullStr(dummyPaymentFlag))
            {
                AddHash(KeyDef.DUMMY_PAYMENT_FLAG, dummyPaymentFlag);
            }

            StringBuilder sb = new StringBuilder();

            // ハッシュテーブルの内容を出力
            foreach (string k in hash.Keys)
            {
                sb.Append(String.Format("{0}={1}&", k, HttpUtility.UrlEncode(hash[k].ToString())));
                 
            }

            sb.Append(sbId).Append(sbJan).Append(sbName).Append(sbUnit).Append(sbNum);

            // 末尾の＆を削除
            sb.Remove(sb.Length-1, 1);


		    logger.Info("送信内容：" + sb.ToString());
		    return sb.ToString();
        }

        /// <summary>
        /// 送信付加情報を設定する
        /// </summary>
        private void addInfomation()
        {
            String type = hash[KeyDef.SETTLEMENT_TYPE].ToString();
            MerchantConf info = MerchantConf.getInfo();
            String tmp = null;
            // 共通
            AddHash(KeyDef.MERCHANT_ID, info.merchantID);

            // VTWebで指定
            if (String.IsNullOrEmpty(type) || Constants.SETTLEMENT_TYPE_NONE.Equals(type))
            {
                // 支払い方法指定無しの場合は、全支払方法に対応できるよう、固有項目をセット
                // 支払取消期限
                int cancelLimit = Constants.DEFAULT_CANCEL_LIMIT_DAYS;
                hash[KeyDef.TIMELIMIT_OF_CANCEL] = DateUtil.getDateYYYYMMDD(cancelLimit);

                // 売り上げフラグ
                hash[KeyDef.CARD_CAPTURE_FLAG] = "1".Equals(info.cardCaptureFlag) ? "1" : "0";

                // SUICA用商品名
                tmp = info.screenTitle;
                if (!String.IsNullOrEmpty(tmp))
                {
                    hash[KeyDef.SCREEN_TITLE] = tmp;
                }

                // ショップ名
                tmp = info.shopName;
                if (!String.IsNullOrEmpty(tmp))
                {
                    hash[KeyDef.SHOP_NAME] = tmp;
                }

                // 請求内容
                tmp = info.contents;
                if (!String.IsNullOrEmpty(tmp))
                {
                    hash[KeyDef.CONTENTS] = tmp;
                }

                // 請求内容カナ
                tmp = info.contentsKana;
                if (!String.IsNullOrEmpty(tmp))
                {
                    hash[KeyDef.CONTENTS_KANA] = tmp;
                }

                // 配送先郵便番号
                tmp = "123-4567";
                hash[KeyDef.SHIPPING_ZIP_CODE] = tmp;

            }

            // カード決済
            if (Constants.SETTLEMENT_TYPE_CARD.Equals(type))
            {
                // カード決済固有項目のセット
                // 売り上げフラグ
                if ("1".Equals(info.cardCaptureFlag))
                {
                    AddHash(KeyDef.CARD_CAPTURE_FLAG, "1");
                }
                else
                {
                    AddHash(KeyDef.CARD_CAPTURE_FLAG, "0");
                }
            }

            // 電子マネー決済
            if (Constants.SETTLEMENT_TYPE_EM.Equals(type))
            {
                // 電子マネー決済固有項目のセット
                String subType = GetHashValue(KeyDef.SETTLEMENT_SUB_TYPE);
                if (StringUtil.isNullStr(subType))
                {
                    // SUICA用商品名
                    tmp = info.screenTitle;
                    if (!StringUtil.isNullStr(tmp))
                    {
                        AddHash(KeyDef.SCREEN_TITLE, tmp);
                    }

                    // 支払取消期限。
                    // この項目はWAONしか使用しない
                    int cancelLimit = Constants.DEFAULT_CANCEL_LIMIT_DAYS;
                    AddHash(KeyDef.TIMELIMIT_OF_CANCEL, DateUtil.getDateYYYYMMDD(cancelLimit));
                }
                else if (Constants.EM_SUBTYPE_EDY_PC.Equals(subType))
                {
                    // 支払期限は10日固定なのでセットしない
                }
                else if (Constants.EM_SUBTYPE_EDY_MOBILE.Equals(subType))
                {
                    tmp = info.shopName;
                    if (!StringUtil.isNullStr(tmp))
                    {
                        AddHash(KeyDef.SHOP_NAME, tmp);
                    }
                }
                else if (Constants.EM_SUBTYPE_SUICA_MOBILE.Equals(subType) || Constants.EM_SUBTYPE_SUICA_PC.Equals(subType))
                {
                    tmp = info.screenTitle;
                    if (!StringUtil.isNullStr(tmp))
                    {
                        AddHash(KeyDef.SCREEN_TITLE, tmp);
                    }
                }
                else if (Constants.EM_SUBTYPE_WAON_PC.Equals(subType) || Constants.EM_SUBTYPE_WAON_MOBILE.Equals(subType))
                {
                    int cancelLimit = Constants.DEFAULT_CANCEL_LIMIT_DAYS;
                    AddHash(KeyDef.TIMELIMIT_OF_CANCEL, DateUtil.getDateYYYYMMDD(cancelLimit));
                }
            }
            // 銀行決済
            if (Constants.SETTLEMENT_TYPE_BANK.Equals(type))
            {
                // 銀行決済固有項目のセット

                // 請求内容
                if (StringUtil.isNullStr(GetHashValue(KeyDef.CONTENTS)))
                {
                    AddHash(KeyDef.CONTENTS, info.contents);
                }

                // 請求内容カナ
                if (StringUtil.isNullStr(GetHashValue(KeyDef.CONTENTS_KANA)))
                {
                    AddHash(KeyDef.CONTENTS_KANA, info.contentsKana);
                }
            }

            AddHash(KeyDef.ORICO_ORDER_NO, createDummyReceiptNo());
        }

        /// <summary>
        /// VT Webに送信する
        /// </summary>
        /// <param name="sessionId">セッションID</param>
        /// <param name="orderId">注文ID</param>
        /// <returns>VT Webからの暗号鍵情報</returns>
        public EncryptionKey send(String sessionId, String orderId)
	    {
		    logger.Info("POST送信開始");
            try
            {
                // VTWEBはクライアント証明書を配布しない
                // だからクライアント証明書のVerifyは必要ない

                MerchantConf info = MerchantConf.getInfo();
                String url = info.vtWebRegistUrl;
                byte[] data = Encoding.ASCII.GetBytes(getPostDataString(sessionId, orderId));
                ServicePointManager.SecurityProtocol = SecurityProtocolType.Tls11 | SecurityProtocolType.Tls12;
                // リクエストの作成
                HttpWebRequest req = (HttpWebRequest)WebRequest.Create(url);
                req.Method = "POST";
                req.ContentType = "application/x-www-form-urlencoded; charset=" + Constants.VTW_ENCODE;
                req.ContentLength = data.Length;

                // ポスト・データの書き込み
                Stream reqStream = req.GetRequestStream();
                reqStream.Write(data, 0, data.Length);
                reqStream.Close();

                // レスポンスの取得と読み込み
                WebResponse res = req.GetResponse();
                Stream resStream = res.GetResponseStream();
                StreamReader sr = new StreamReader(resStream, Encoding.UTF8);

                // 一行ずつ読み込みます
                EncryptionKey ret = new EncryptionKey();
                String line;
                try
                {
                    while ((line = sr.ReadLine()) != null)
                    {
                        if (String.IsNullOrEmpty(line))
                            continue;

                        // ログに残します
                        logger.Info("返信内容：" + line);

                        if (line.StartsWith("MERCHANT_ENCRYPTION_KEY"))
                        {
                            String tmp = line.Substring(24);
                            ret.merchantEncryptionKey = tmp;
                        }
                        else if (line.StartsWith("BROWSER_ENCRYPTION_KEY"))
                        {
                            String tmp = line.Substring(23);
                            ret.browserEncryptionKey = tmp;
                        }
                        else if (line.StartsWith("ERROR_MESSAGE"))
                        {
                            String tmp = HttpUtility.UrlDecode(line.Substring(14), Encoding.UTF8);
                            ret.errorMessage = tmp;
                        }
                    }
                }
                catch (Exception ex)
                {
                    logger.Error("受信結果解析エラーです", ex);
                    throw ex;
                }
                finally
                {
                    // 入力ストリームを閉じます
                    sr.Close();
                    resStream.Close();

                    logger.Info("POST通信完了");
                }

                return ret;

            }
            catch (Exception ex)
            {
                logger.Error("接続エラーです。", ex);
                throw ex;
            }
	    }

        /// <summary>
        /// 注文番号を作成する
        /// </summary>
        /// <returns>作成した注文番号</returns>
        protected static String createDummyReceiptNo()
        {
            // 注文番号を作成(現在時刻をミリ秒まで)
            String receiptNo = DateTime.Now.ToString("yyyyMMddHHmmssSSS") + String.Format("000",(new Random()).Next(0, 999));
            return receiptNo;
        }
    }
}