﻿using System;
using System.Collections.Generic;
using System.Web;
using System.Text.RegularExpressions;

namespace MDK
{
    /// <summary>
    /// 文字列ユーティリティクラス
    /// </summary>
    public class StringUtil
    {
        /// <summary>
        /// コンストラクタ
        /// </summary>
	    public StringUtil()
	    {
	    }

        /// <summary>
        /// ログ出力用インスタンス
        /// </summary>
        private static readonly log4net.ILog logger =
            log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

	    /// <summary>
        /// デフォルト改行コード
	    /// </summary>
	    private static String CRLF = "\r\n";

	    /// <summary>
        /// 半角数字チェック
	    /// </summary>
	    /// <param name="str">チェック文字列</param>
	    /// <returns>チェック結果</returns>
	    public static bool isSingleByteNumeric(String str) {
		    if (!isNullStr(str)) {
			    return Regex.IsMatch(str, "^[0-9]+$");
		    }
		    return false;
	    }

	    /// <summary>
        /// チェック文字列がNULL又は空文字列であるか否かを判定する。
	    /// </summary>
	    /// <param name="str">チェック文字列</param>
	    /// <returns>チェック結果</returns>
	    public static bool isNullStr(String str) {
		    if (str == null || str.Length == 0) {
			    return true;
		    } else {
			    return false;
		    }
	    }

	    /// <summary>
        /// 英数字チェック
	    /// </summary>
	    /// <param name="str">チェック文字列</param>
	    /// <returns>チェック結果</returns>
	    public static bool isAlNum(String str) {
		    if (Regex.IsMatch(str, "[^a-zA-Z0-9]")) {
			    return false;
		    } else {
			    return true;
		    }
	    }

        /// <summary>
        /// 数字チェック
        /// </summary>
        /// <param name="str">チェック文字列</param>
        /// <returns>チェック結果</returns>
	    public static bool isNumeric(String str) {
		    try {
                long.Parse(str);
		    } catch {
			    return false;
		    }
		    return true;
	    }

        /// <summary>
        /// 半角英数字記号
        /// </summary>
        /// <param name="str">チェック文字列</param>
        /// <returns>チェック結果</returns>
	    public static bool isAlnumSymbol(String str) {
            if (Regex.IsMatch(str, "^[\x20-\x7F]+$"))
            {
			    return true;
		    } else {
			    return false;
		    }
	    }

        /// <summary>
        /// 全角カナチェック
        /// </summary>
        /// <param name="str">チェック文字列</param>
        /// <returns>チェック結果</returns>
	    public static bool isMultiByteKana(String str) {
            if (Regex.IsMatch(str, "^[ァ-ー]+$"))
            {
			    return true;
		    } else {
			    return false;
		    }
	    }

        /// <summary>
        /// 処理対象文字列がNULLであれば空文字列を返却する。
        /// </summary>
        /// <param name="str">処理対象文字列</param>
        /// <returns>処理結果</returns>
	    public static String emptyString(String str) {
		    if (str == null) {
			    return "";
		    } else {
			    return str;
		    }
	    }

	    /// <summary>
        /// 全角チェック
	    /// </summary>
	    /// <param name="str">チェック文字列</param>
	    /// <returns>チェック結果</returns>
	    public static bool isMultiByte(String str) {
		    if (!isNullStr(str)) {
			    return Regex.IsMatch(str, "^[^ -~｡-ﾟ]+$");
		    }
		    return false;
	    }

        /// <summary>
        /// 半角チェック
        /// </summary>
        /// <param name="str">チェック文字列</param>
        /// <returns>チェック結果</returns>
	    public static bool isSingleByte(String str) {
		    if (!isNullStr(str)) {
			    return Regex.IsMatch(str, "^[ -~｡-ﾟ]+$");
		    }
		    return false;
	    }

        /// <summary>
        /// 改行コード取得
        /// </summary>
        /// <returns>改行コード</returns>
	    public static String getCRLF() {
		    String retStr = null;
		    try {
                String lineSep = Environment.NewLine;
			    if (lineSep == null || lineSep.Length == 0) {
				    retStr = CRLF;
			    } else {
				    retStr = lineSep;
			    }
		    } catch (Exception ex) {
                logger.Error("改行コード(Environment.NewLine)の取得に失敗しました。", ex);
			    retStr = CRLF;
		    }
		    return retStr;
	    }

        /// <summary>
        /// カード番号で入力されない文字列を削除する
        /// </summary>
        /// <param name="cardNumber">処理対象文字列</param>
        /// <returns>処理結果</returns>
	    public static String removeForbidChar4CardNumber(String cardNumber) {
		    String retStr = "";
		    if (!isNullStr(cardNumber)) {
			    retStr = cardNumber.Replace("[^0-9\\-]", "");
		    }
		    return retStr;
	    }

        /// <summary>
        /// 半角数字以外を削除する
        /// </summary>
        /// <param name="str">処理対象文字列</param>
        /// <returns>処理結果</returns>
	    public static String removeNonNumber(String str) {
		    String retStr = "";
		    if (!isNullStr(str)) {
			    retStr = str.Replace("[^0-9]", "");
		    }
		    return retStr;
	    }
    }
}