﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using MDK;

namespace VTWebNET
{
    /// <summary>
    /// 決済情報設定画面
    /// </summary>
    public partial class IfPurchase : System.Web.UI.Page
    {
        private SendPostData postData;

        /** 　ログ出力用インスタンス　 */
        private static readonly log4net.ILog logger =
            log4net.LogManager.GetLogger(System.Reflection.MethodBase.GetCurrentMethod().DeclaringType);

        /// <summary>
        /// ページロード時処理
        /// </summary>
        /// <param name="sender">object</param>
        /// <param name="e">EventArgs</param>
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!IsPostBack)
            {
                ///////////////////////////////////////////////////////////
                ///////////////////////////////////////////////////////////
                // テスト用にCommodityをセッションに登録する
                // 本来は、このページに来た時点で、既に登録されているはず

                List<CommodityDetail> listCD = new List<CommodityDetail>();

                CommodityDetail cd = null;

                cd = new CommodityDetail("", "", "天然水", 100, 1);
                listCD.Add(cd);
                cd = new CommodityDetail(null, null, "イオン水", 200, 2);
                listCD.Add(cd);
                cd = new CommodityDetail("003", "J03", "電解水", 300, 3);
                listCD.Add(cd);
                Session["ShippingAmount"] = 500;
                Session["ListCommodityDetail"] = listCD;
                ///////////////////////////////////////////////////////////
                ///////////////////////////////////////////////////////////

                postData = new SendPostData();

                // Commodityをセッションから取得
                postData.setListCommodityDetail((List<CommodityDetail>)Session["ListCommodityDetail"]);
                postData.ShippingAmount = (int)Session["ShippingAmount"];
                Session["SendPostData"] = postData;
            }
            else
            {
                postData = (SendPostData)Session["SendPostData"];
                lblErrMsg.Text = String.Empty;
            }

            // Cartテーブルの作成
            makeCartTable();
        }

        /// <summary>
        /// 商品テーブルに表示する情報を作成
        /// </summary>
        protected void makeCartTable()
        {
            int productAmount = 0;
            foreach (CommodityDetail cd in (List<CommodityDetail>)Session["ListCommodityDetail"])
            {
                TableRow tRow = new TableRow();
                TableCell tCell;

                // 商品名
                tCell = new TableCell();
                tCell.Text = cd.commodityName;
                tCell.CssClass = "name";
                tRow.Controls.Add(tCell);

                // 単価
                tCell = new TableCell();
                tCell.Text = cd.commodityUnit.ToString() + " 円";
                tCell.CssClass = "unitp";
                tRow.Controls.Add(tCell);

                // 個数
                tCell = new TableCell();
                tCell.Text = cd.commodityNum.ToString() + " 個";
                tCell.CssClass = "num";
                tRow.Controls.Add(tCell);

                // 小計
                tCell = new TableCell();
                tCell.Text = cd.commodityUnit * cd.commodityNum + " 円";
                tCell.CssClass = "sum";
                tRow.Controls.Add(tCell);
                productAmount += cd.commodityUnit * cd.commodityNum;

                tblCart.Controls.Add(tRow);
            }
            TableRow saRow = new TableRow();
            TableCell saCell = new TableCell();
            saCell.Text = "送料合計 : " + Session["ShippingAmount"] + " 円";
            saCell.ColumnSpan = 4;
            saCell.HorizontalAlign = HorizontalAlign.Right;
            saRow.Controls.Add(saCell);

            tblCart.Controls.Add(saRow);

            TableRow paRow = new TableRow();
            TableCell paCell = new TableCell();
            int amount = productAmount + Convert.ToInt32(Session["ShippingAmount"]);
            paCell.Text = "合計 : " + amount + " 円";
            paCell.ColumnSpan = 4;
            paCell.HorizontalAlign = HorizontalAlign.Right;
            paRow.Controls.Add(paCell);

            tblCart.Controls.Add(paRow);
        }

        /// <summary>
        /// 「後で指定」リンククリック時の処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbDefault_Click(object sender, EventArgs e)
        {
            pnlChange();
            postData.AddHash(KeyDef.SETTLEMENT_TYPE, Constants.SETTLEMENT_TYPE_NONE);
        }

        /// <summary>
        /// 「カード」リンククリック時の処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbCard_Click(object sender, EventArgs e)
        {
            pnlChange();
            postData.AddHash(KeyDef.SETTLEMENT_TYPE, Constants.SETTLEMENT_TYPE_CARD);
            makeSettlementTable_Card();
        }

        /// <summary>
        /// 「ショッピングクレジット」リンククリック時の処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbOricoSC_Click(object sender, EventArgs e)
        {
            pnlChange();
            postData.AddHash(KeyDef.SETTLEMENT_TYPE, Constants.SETTLEMENT_TYPE_ORICOSC);
            makeSettlementTable_OricoSC();
        }

        /// <summary>
        /// 「コンビニ」リンククリック時の処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbCvs_Click(object sender, EventArgs e)
        {
            pnlChange();
            postData.AddHash(KeyDef.SETTLEMENT_TYPE, Constants.SETTLEMENT_TYPE_CVS);
            makeSettlementTable_Cvs();
        }

        /// <summary>
        /// 「電子マネー」リンククリック時の処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbEm_Click(object sender, EventArgs e)
        {
            pnlChange();
            postData.AddHash(KeyDef.SETTLEMENT_TYPE, Constants.SETTLEMENT_TYPE_EM);
            makeSettlementTable_Em();
        }

        /// <summary>
        /// 「銀行」リンククリック時の処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbBank_Click(object sender, EventArgs e)
        {
            pnlChange();
            postData.AddHash(KeyDef.SETTLEMENT_TYPE, Constants.SETTLEMENT_TYPE_BANK);
            makeSettlementTable_Bank();
        }

        /// <summary>
        /// 「銀行振込」リンククリック時の処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void lbVacc_Click(object sender, EventArgs e)
        {
            pnlChange();
            postData.AddHash(KeyDef.SETTLEMENT_TYPE, Constants.SETTLEMENT_TYPE_VACC);
            makeSettlementTable_Vacc();
        }

        /// <summary>
        /// 決済方法リンククリック時の共通処理
        /// </summary>
        protected void pnlChange()
        {
            H1.Visible = true;
            pnlEntrySelect.Visible = false;
            pnlSettlement.Visible = true;
        }

        /// <summary>
        /// 「カード」リンククリック時の処理
        /// </summary>
        protected void makeSettlementTable_Card()
        {
            lblSettlementType.Text = "カード決済";
        }

        /// <summary>
        /// 「ショッピングクレジット」リンククリック時の処理
        /// </summary>
        protected void makeSettlementTable_OricoSC()
        {
            lblSettlementType.Text = "ショッピングクレジット決済";
            trOricoscSettlementSubType.Visible = true;
        }

        /// <summary>
        /// 「コンビニ」リンククリック時の処理
        /// </summary>
        protected void makeSettlementTable_Cvs()
        {
            lblSettlementType.Text = "コンビニ決済";
            trCvsSettlementSubType.Visible = true;
        }

        /// <summary>
        /// 「電子マネー」リンククリック時の処理
        /// </summary>
        protected void makeSettlementTable_Em()
        {
            lblSettlementType.Text = "電子マネー決済";
            trEmSettlementSubType.Visible = true;
        }

        /// <summary>
        /// 「銀行」リンククリック時の処理
        /// </summary>
        protected void makeSettlementTable_Bank()
        {
            lblSettlementType.Text = "銀行決済";
            trBankSettlementSubType.Visible = true;
        }

        /// <summary>
        /// 「銀行振込」リンククリック時の処理
        /// </summary>
        protected void makeSettlementTable_Vacc()
        {
            lblSettlementType.Text = "銀行振込決済";
            vaccPaynowid.Visible = true;
            trVaccAccountManageType.Visible = true;
            trVaccAccountNumber.Visible = true;
            trVaccBranchCode.Visible = true;
            trVaccEntryTransferNumber.Visible = true;
            trVaccRejectBankAccount.Visible = true;
            trVaccTransferExpiredDate.Visible = true;
        }

        /// <summary>
        /// 「送信」ボタンクリック時の処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        protected void btnSubmit_Click(object sender, EventArgs e)
        {
            lblMainErrMsg.Visible = true;
            String type = postData.GetHashValue(KeyDef.SETTLEMENT_TYPE);

            postData.AddHash(KeyDef.SETTLEMENT_SUB_TYPE, "");

            if (type.Equals(Constants.SETTLEMENT_TYPE_NONE))
            {
            }
            else if (type.Equals(Constants.SETTLEMENT_TYPE_CARD))
            {
            }
            else if (type.Equals(Constants.SETTLEMENT_TYPE_ORICOSC))
            {
                if (!isValidSettlement_Oricosc())
                {
                    return;
                }
            }
            else if (type.Equals(Constants.SETTLEMENT_TYPE_CVS))
            {
                postData.AddHash(KeyDef.SETTLEMENT_SUB_TYPE, ddlCvsSettlementSubType.SelectedValue);
            }
            else if (type.Equals(Constants.SETTLEMENT_TYPE_EM))
            {
                postData.AddHash(KeyDef.SETTLEMENT_SUB_TYPE, ddlEmSettlementSubType.SelectedValue);
            }
            else if (type.Equals(Constants.SETTLEMENT_TYPE_BANK))
            {
                postData.AddHash(KeyDef.SETTLEMENT_SUB_TYPE, ddlBankSettlementSubType.SelectedValue);
            }
            else if (type.Equals(Constants.SETTLEMENT_TYPE_VACC))
            {
                if (!isValidSettlement_Vacc())
                {
                    return;
                }
                if (cbPaynowidFlag.Checked)
                {
                    postData.AddHash(KeyDef.PAYNOWID_FLAG, "1");
                    postData.AddHash(KeyDef.ACCOUNT_ID, tbAccountId.Text);
                }
                else
                    postData.AddHash(KeyDef.PAYNOWID_FLAG, "0");
                if (cbRejectBankAccount.Checked)
                {
                    postData.AddHash(KeyDef.VACC_REJECT_BANK_ACCOUNT, "1");
                }
                else
                    postData.AddHash(KeyDef.VACC_REJECT_BANK_ACCOUNT, "0");
                    postData.AddHash(KeyDef.VACC_ENTRY_TRANSFER_NUMBER, tbEntryTransferNumber.Text);
                    postData.AddHash(KeyDef.VACC_TRANSFER_EXPIRED_DATE, tbTransferExpiredDate.Text);
                    postData.AddHash(KeyDef.VACC_ACCOUNT_MANAGE_TYPE, rblAccountManageType.SelectedValue);
                    postData.AddHash(KeyDef.VACC_BRANCH_CODE, tbBranchCode.Text);
                    postData.AddHash(KeyDef.VACC_ACCOUNT_NUMBER, tbAccountNumber.Text);
            }

            // ここまででPOSTデータのチェックとハッシュへの入力は終了

            // VTWeb に Request
            try
            {
                // テスト用にダミーの 取引IDを作成して使用する
                String orderId = MDKUtil.createDummyOrderId();
                EncryptionKey encKey = postData.send(Session.SessionID, orderId);

                // 成功か失敗か？
                if (encKey.errorMessage != null && encKey.errorMessage.Length != 0)
                    addError("VTWeb:" + encKey.errorMessage);
                else
                {
                    lblMainErrMsg.Visible = false;
                    // セッションに保存して遷移
                    Session["orderId"] = orderId;
                    Session["EncryptionKey"] = encKey;
                    Response.Redirect("~/IfJump.aspx");
                }

            }
            catch (Exception ex)
            {
                addError(ex.Message);
            }
        }

        /// <summary>
        /// ショッピングクレジット決済時の入力項目チェック
        /// </summary>
        /// <returns></returns>
        protected bool isValidSettlement_Oricosc()
        {
            bool retVal = true;

            String shippingZipCodeText = shippingZipCode1.Text + "-" + shippingZipCode2.Text;
            if (!String.IsNullOrEmpty(shippingZipCode1.Text) && !String.IsNullOrEmpty(shippingZipCode2.Text))
            {
                if (!isValidShippingCode(shippingZipCode1.Text, shippingZipCode2.Text))
                {
                    addError(MessageResources.ERR_SHIPPINGCODE_INVALID);
                    retVal = false;
                }
                else
                {
                    postData.AddHash(KeyDef.SHIPPING_ZIP_CODE, shippingZipCodeText);
                }
            }
            return retVal;
        }

        /// <summary>
        /// 銀行振込決済時の入力項目チェック
        /// </summary>
        /// <returns></returns>
        protected bool isValidSettlement_Vacc()
        {
            bool retVal = true;
            String str = String.Empty;
            // 支店コード
            str = tbBranchCode.Text;
            if (!StringUtil.isNullStr(str) && !StringUtil.isNumeric(str))
            {
                addError(MessageResources.ERR_BRANCHCODE_INVALID);
                retVal = false;
            }

            // 口座番号
            str = tbAccountNumber.Text;
            if (!StringUtil.isNullStr(str) && !StringUtil.isNumeric(str))
            {
                addError(MessageResources.ERR_ACCOUNTNUMBER_INVALID);
                retVal = false;
            }

            return retVal;
        }

        /// <summary>
        /// エラーメッセージの追加
        /// </summary>
        /// <param name="str">エラーメッセージ</param>
        protected void addError(String str)
        {
            lblErrMsg.Text += str + "<br />";
        }

        /// <summary>
        /// 配送先郵便番号チェック
        /// </summary>
        /// <param name="shippingCode1"></param>
        /// <param name="shippingCode2"></param>
        /// <returns>チェック結果</returns>
        private bool isValidShippingCode(String shippingCode1, String shippingCode2)
        {
            if (!StringUtil.isNumeric(shippingCode1) || !StringUtil.isNumeric(shippingCode1))
            {
                return false;
            }
            return true;
        }
    }
}
